from distutils.core import setup, Extension

module1 = Extension('_util',
                    sources = ['_util.c'])

setup (name = '_util',
       version = '0.1',
       description = 'GEOS util package',
       ext_modules = [module1])
